# generated by datamodel-codegen:
#   filename:  model.json
#   timestamp: 2021-04-11T02:41:02+00:00

from __future__ import annotations

from datetime import date
from enum import Enum
from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class Type(Enum):
    order_created = 'order.created'
    order_updated = 'order.updated'
    order_deleted = 'order.deleted'


class PaymentProcessor(Enum):
    PAYPAL = 'PAYPAL'
    STRIPE = 'STRIPE'


class User(BaseModel):
    discord_id: Optional[UUID] = None
    username: Optional[str] = None


class Type1(Enum):
    UPGRADE = 'UPGRADE'
    SHOP = 'SHOP'


class Interval(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'


class Type2(Enum):
    UPGRADE = 'UPGRADE'
    SHOP = 'SHOP'


class DiscordRole(BaseModel):
    discord_id: Optional[str] = None
    name: Optional[str] = None


class ProductType(Enum):
    DISCORD_ROLE = 'DISCORD_ROLE'
    SHOP_PRODUCT = 'SHOP_PRODUCT'


class Product(BaseModel):
    uuid: Optional[UUID] = None
    name: Optional[str] = None


class OrderItem(BaseModel):
    price: Optional[float] = None
    quantity: Optional[float] = None
    interval: Optional[Interval] = None
    interval_count: Optional[float] = None
    free_trial_length: Optional[float] = None
    is_time_limited: Optional[bool] = None
    type: Optional[Type2] = None
    discord_roles: Optional[List[DiscordRole]] = None
    product_types: Optional[List[ProductType]] = Field(
        None,
        description='The types of the product. A product purchased through the shop will be a shop product. All other types are upgrades.',
    )
    product: Optional[Product] = Field(None, description='An Upgrade.Chat product')


class Body(BaseModel):
    uuid: Optional[str] = None
    purchased_at: Optional[date] = None
    payment_processor: Optional[PaymentProcessor] = None
    payment_processor_record_id: Optional[str] = None
    user: Optional[User] = None
    subtotal: Optional[float] = None
    discount: Optional[float] = None
    total: Optional[float] = None
    type: Optional[Type1] = None
    is_subscription: Optional[bool] = None
    cancelled_at: Optional[date] = Field(
        None, description='The date when the subscription was cancelled'
    )
    deleted: Optional[date] = Field(
        None, description='The date when the subscription expired'
    )
    order_items: Optional[List[OrderItem]] = None


class Datum(BaseModel):
    id: Optional[UUID] = None
    webhook_id: Optional[UUID] = None
    type: Optional[Type] = None
    body: Optional[Body] = None
    attempts: Optional[float] = None


class Model(BaseModel):
    data: Optional[List[Datum]] = Field(
        None, description='An array of webhook event records'
    )
    total: Optional[float] = Field(None, description='Total number of records')
    has_more: Optional[bool] = Field(
        None, description='Boolean indicating if there are more records.'
    )

    def __str__(self):
        return f"<{len(self.data)} Webhook Events>"



from typing import Union

from upgradeChat.request_handler import RequestHandler
from upgradeChat.models import *


# TODO: __str__ for all model objects and sub-objects

class API:
    def __init__(self, client_id: str, client_secret: str, raw: bool = False):
        self._make_objects = not raw
        self._request_handler = RequestHandler(client_id=client_id,
                                               client_secret=client_secret,
                                               base_url="https://api.upgrade.chat/v1")

    def _request(self, uri: str, params: dict = None, model: type = None) -> Union[object, dict]:
        if self._make_objects:
            return self._request_handler.get_object(uri=uri, model=model, params=params)
        return self._request_handler.get_json(uri=uri, params=params)

    # Orders
    def get_orders(self, order_type: str = None, discord_id: int = None, limit: int = 100, offset: int = 0) -> Union[
        Orders, dict]:
        """
        Get a list of orders

        :param order_type: filter to a specific order type
        :type order_type: str
        :param discord_id: filter to a specific Discord user
        :type discord_id: int
        :param limit: limit the number of items returned (default: 100)
        :type limit: int
        :param offset: offset the number of items returned (default: 0)
        :type offset: int
        :return: Orders object or JSON
        :rtype: Orders | dict
        """
        params = {
            'limit': limit,
            'offset': offset,
        }
        if discord_id:
            params['userDiscordId'] = discord_id
        if order_type:
            params['type'] = order_type
        return self._request(uri='orders', params=params, model=Orders)

    @property
    def orders(self) -> Union[Orders, dict]:
        """
        Get all orders (up to 100)

        :return: Orders object or JSON
        :rtype: Orders | dict
        """
        return self.get_orders()

    def orders_for_user(self, discord_id: int) -> Union[Orders, dict]:
        """
        Get a list of orders for a specific Discord user

        :param discord_id: Discord ID of user
        :type discord_id: int
        :return: Orders object or JSON
        :rtype: Orders | dict
        """
        return self.get_orders(discord_id=discord_id)

    def get_order(self, order_id: str) -> Union[Order, dict]:
        """
        Get a specific order

        :param order_id: ID of the order
        :type order_id: str
        :return: Order object or JSON
        :rtype: Order | dict
        """
        return self._request(uri=f"orders/{order_id}", model=Order)

    # Products
    def get_products(self, product_type: str = None, limit: int = 100, offset: int = 0) -> Union[Products, dict]:
        """
        Get a list of products

        :param product_type: filter to a specific product type
        :type product_type: str
        :param limit: limit the number of items returned (default: 100)
        :type limit: int
        :param offset: offset the number of items returned (default: 0)
        :type offset: int
        :return: Products object or JSON
        :rtype: Products | dict
        """
        params = {
            'limit': limit,
            'offset': offset
        }
        if product_type:
            params['type'] = product_type
        return self._request(uri=f"products", params=params, model=Products)

    @property
    def products(self) -> Union[Products, dict]:
        """
        Get all products (up to 100)

        :return: Products object or JSON
        :rtype: Products | dict
        """
        return self.get_products()

    def get_product(self, product_id: str) -> Union[Product, dict]:
        """
        Get a specific product

        :param product_id: ID of the product
        :type product_id: str
        :return: Product object or JSON
        :rtype: Product | dict
        """
        return self._request(uri=f"products/{product_id}", model=Product)

    # Users
    def get_users(self, limit: int = 100, offset: int = 0) -> Union[Users, dict]:
        """
        Get a list of users

        :param limit: limit the number of items returned (default: 100)
        :type limit: int
        :param offset: offset the number of items returned (default: 0)
        :type offset: int
        :return: Users object or JSON
        :rtype: Users | dict
        """
        params = {
            'limit': limit,
            'offset': offset
        }
        return self._request(uri=f"users", params=params, model=Users)

    @property
    def users(self) -> Union[Users, dict]:
        """
        Get all users (up to 100)

        :return: Users object or JSON
        :rtype: Users | dict
        """
        return self.get_users()

    # Webhooks
    def get_webhooks(self, limit: int = 100, offset: int = 0) -> Union[Webhooks, dict]:
        """
        Get a list of webhooks

        :param limit: limit the number of items returned (default: 100)
        :type limit: int
        :param offset: offset the number of items returned (default: 0)
        :type offset: int
        :return: Webhooks object or JSON
        :rtype: Webhooks | dict
        """
        params = {
            'limit': limit,
            'offset': offset
        }
        return self._request(uri=f"webhooks", params=params, model=Webhooks)

    @property
    def webhooks(self) -> Union[Webhooks, dict]:
        """
        Get all webhooks (up to 100)

        :return: Webhooks object or JSON
        :rtype: Webhooks | dict
        """
        return self.get_webhooks()

    def get_webhook(self, webhook_id: str) -> Union[Webhook, dict]:
        """
        Get a specific webhook

        :param webhook_id: ID of the webhook
        :type webhook_id: str
        :return: Webhook object or JSON
        :rtype: Webhook | dict
        """
        return self._request(uri=f"webhooks/{webhook_id}", model=Webhook)

    # Webhook Events
    def get_webhook_events(self, limit: int = 100, offset: int = 0) -> Union[WebookEvents, dict]:
        """
        Get a list of webhook events

        :param limit: limit the number of items returned (default: 100)
        :type limit: int
        :param offset: offset the number of items returned (default: 0)
        :type offset: int
        :return: WebhookEvents object or JSON
        :rtype: WebhookEvents | dict
        """
        params = {
            'limit': limit,
            'offset': offset
        }
        return self._request(uri=f"webhook-events", params=params, model=WebookEvents)

    @property
    def webhook_events(self) -> Union[WebookEvents, dict]:
        """
        Get all webhook events (up to 100)

        :return: WebhookEvents object or JSON
        :rtype: WebhookEvents | dict
        """
        return self.get_webhook_events()

    def get_webhook_event(self, webhook_event_id: str) -> Union[WebhookEvent, dict]:
        """
        Get a specific webhook event

        :param webhook_event_id: ID of the webhook event
        :type webhook_event_id: str
        :return: WebhookEvent object or JSON
        :rtype: WebhookEvent | dict
        """
        return self._request(uri=f"webhook-events/{webhook_event_id}", model=WebhookEvent)

    def validate_webhook_event(self, webhook_event_id: str) -> bool:
        """
        Validate a webhook event

        :param webhook_event_id: ID of the webhook event
        :type webhook_event_id: str
        :return: True or False
        :rtype: bool
        """
        data = self._request_handler.get_json(uri=f"webhook-events/{webhook_event_id}/validate")
        return data.get('valid', False)



import time

def timestamp_is_expired(timestamp: str):
    try:
        timestamp = int(timestamp)
        current_timestamp = int(time.time())
        return timestamp < current_timestamp
    except:
        raise Exception("Could not check timestamp expiration.")


import requests
from requests_oauthlib import OAuth2Session
from requests.auth import HTTPBasicAuth
from oauthlib.oauth2 import BackendApplicationClient

from objectrest import requests as objectrest_requests
from upgradeChat import utils


class RequestHandler:
    def __init__(self, base_url: str, client_id: str, client_secret: str):
        if base_url.endswith("/"):
            base_url = base_url[:-1]
        self._base = base_url
        self._id = client_id
        self._secret = client_secret

        self._tokens = self._authorize()
        self._session = requests.Session()

    def _authorize(self):
        """
        Access token seems to be valid for a month
        Refresh token seems to be valid for a year

        :return:
        :rtype:
        """
        url = "https://api.upgrade.chat/oauth/token"
        auth = HTTPBasicAuth(self._id, self._secret)
        client = BackendApplicationClient(client_id=self._id)
        oauth = OAuth2Session(client=client)
        return oauth.fetch_token(token_url=url, auth=auth)

    def _get_access_token(self):
        """
        Handle refreshing tokens if needed

        :return:
        :rtype:
        """
        # If not authorized already
        if not self._tokens:
            self._authorize()

        # If access token has expired
        access_token_expiration_timestamp = self._tokens.get('access_token_expires_in')
        if not access_token_expiration_timestamp \
                or utils.timestamp_is_expired(timestamp=access_token_expiration_timestamp):
            self._authorize()

        if not self._tokens:
            raise Exception("Could not get tokens from the API.")

        access_token = self._tokens.get('access_token')
        if not access_token:
            raise Exception("No access token provided by the API.")
        return access_token

    def _get_headers(self):
        access_token = self._get_access_token()
        return {'Authorization': f'Bearer {access_token}'}

    def _make_url(self, uri: str):
        if uri.startswith("/"):
            uri = uri[1:]
        return f"{self._base}/{uri}"

    def get_json(self, uri: str, params: dict = None):
        """
        Get API response as JSON

        :param uri:
        :type uri:
        :param params:
        :type params:
        :return:
        :rtype:
        """
        url = self._make_url(uri)
        headers = self._get_headers()
        return objectrest_requests.get_json(url=url, session=self._session, params=params, headers=headers)

    def get_object(self, uri: str, model: type, params: dict = None):
        """
        Get API response as an object

        :param uri:
        :type uri:
        :param params:
        :type params:
        :return:
        :rtype:
        """
        url = self._make_url(uri)
        headers = self._get_headers()
        return objectrest_requests.get_object(url=url, model=model, session=self._session, params=params, headers=headers)



from upgradeChat.models.order import Model as Order
from upgradeChat.models.orders import Model as Orders
from upgradeChat.models.product import Model as Product
from upgradeChat.models.products import Model as Products
from upgradeChat.models.user import Model as User
from upgradeChat.models.users import Model as Users
from upgradeChat.models.webhook import Model as Webhook
from upgradeChat.models.webhooks import Model as Webhooks
from upgradeChat.models.webhook_event import Model as WebhookEvent
from upgradeChat.models.webhook_events import Model as WebookEvents


# generated by datamodel-codegen:
#   filename:  model.json
#   timestamp: 2021-04-11T02:18:22+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel


class User(BaseModel):
    discord_id: str
    username: str


class Product(BaseModel):
    uuid: str
    name: str


class DiscordRole(BaseModel):
    discord_id: str
    name: str


class OrderItem(BaseModel):
    price: int
    interval: str
    interval_count: int
    is_time_limited: bool
    free_trial_length: int
    payment_processor_record_id: str
    payment_processor: str
    quantity: int
    product: Product
    product_types: List[str]
    discord_roles: List[DiscordRole]


class Order(BaseModel):
    uuid: str
    id: str
    purchased_at: str
    payment_processor: str
    payment_processor_record_id: str
    subtotal: int
    total: int
    discount: int
    type: str
    is_subscription: bool
    cancelled_at: str
    deleted: str
    user: User
    order_items: List[OrderItem]


class Model(BaseModel):
    data: Order

    def __str__(self):
        return f"<Order {self.data.uuid}>"



# generated by datamodel-codegen:
#   filename:  model.json
#   timestamp: 2021-04-11T02:36:08+00:00

from __future__ import annotations

from typing import List, Optional
from uuid import UUID

from pydantic import AnyUrl, BaseModel, Field


class Webhook(BaseModel):
    id: Optional[UUID] = None
    uri: Optional[AnyUrl] = None


class Model(BaseModel):
    data: Optional[Webhook] = Field(None)

    def __str__(self):
        return f"<Webhook {self.data.id}>"



# generated by datamodel-codegen:
#   filename:  model.json
#   timestamp: 2021-04-11T02:34:32+00:00

from __future__ import annotations

from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class User(BaseModel):
    discord_id: str
    username: str


class Model(BaseModel):
    data: User

    def __str__(self):
        return f"<User {self.data.username}>"



from upgradeChat.api import API

__version__ = '0.1.1'

__title__ = "upgradeChat"
__github_repo__ = "upgradeChat"
__author__ = 'Nate Harris'
__author_email__ = 'n8gr8gbln@gmail.com'
__github_username__ = "nwithan8"
__copyright__ = "Copyright © 2021 - Nate Harris"
__license__ = 'GNU General Public License v3 (GPLv3)'
__description__ = "A complete Python client for Upgrade.Chat's API"
__keywords__ = ["Discord", "chat", "bot", "Upgrade.Chat", "donations", "subscriptions", "rewards", "management", "API", "client"]


# generated by datamodel-codegen:
#   filename:  model.json
#   timestamp: 2021-04-11T02:21:44+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import List, Optional
from uuid import UUID

from pydantic import AnyUrl, BaseModel, Field


class Interval(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'


class ProductType(Enum):
    DISCORD_ROLE = 'DISCORD_ROLE'
    SHOP_PRODUCT = 'SHOP_PRODUCT'


class Product(BaseModel):
    uuid: Optional[UUID] = None
    checkout_uri: Optional[AnyUrl] = Field(None, description='Direct Link to Product')
    name: Optional[str] = None
    account_id: Optional[float] = None
    price: Optional[float] = None
    interval: Optional[Interval] = None
    interval_count: Optional[int] = None
    free_trial_length: Optional[float] = None
    description: Optional[str] = None
    image_link: Optional[str] = None
    variable_price: Optional[bool] = None
    is_time_limited: Optional[bool] = None
    limited_inventory: Optional[bool] = None
    available_stock: Optional[float] = None
    shippable: Optional[bool] = None
    paymentless_trial: Optional[bool] = None
    product_types: Optional[List[ProductType]] = None
    created: Optional[datetime] = None
    updated: Optional[datetime] = None
    deleted: Optional[datetime] = None

class Model(BaseModel):
    data: Product

    def __str__(self):
        return f"<Product {self.data.uuid}>"
